// Digital Audio Monitor
// version 1.1
// Copyright (C) 2006 by Yasuo Kuwahara

//`define TEST
`define MSB		23

module damon(clk, mclk, rx, up_n, down_n, lo, ro, mclk0, led);
	input clk, mclk, rx, up_n, down_n;
	output [`MSB:0] lo, ro;
	output mclk0, led;
	assign led = 1;
`ifdef TEST
	reg [14:0] cnt;
	wire msb = cnt[14];
	always @(posedge clk) cnt <= cnt + 1;
	assign lo = { ~msb, { `MSB { msb }}};
	assign ro = { ~msb, { `MSB { msb }}};
`else
	reg [9:0] vol;
	wire timing;
	always @(posedge clk) 
		if (timing) 
			if (~up_n & ~&vol) vol <= vol + 1;
			else if (~down_n & |vol) vol <= vol - 1;
	dairvol dairvol(.clk(clk), .mclk(mclk), .rx(rx), .vol(vol[9:1]), 
		.lo(lo), .ro(ro), .timing(timing), .mclk0(mclk0));
`endif
endmodule

module dairvol(clk, mclk, rx, vol, lo, ro, timing, mclk0);
	input clk, mclk, rx;
	input [8:0] vol;
	output [`MSB:0] lo, ro;
	output timing, mclk0;
	reg [2:0] rxs;
	always @(posedge clk) rxs <= { rxs[1:0], rx };
	wire rxedge = ^rxs[2:1];
	reg [3:0] clkcnt;
	wire mute = &clkcnt;
	always @(posedge clk) 
		if (rxedge) clkcnt <= 0;
		else if (~mute) clkcnt <= clkcnt + 1;
	reg [5:0] ta;
	wire [7:0] td = { ta, clkcnt > 9, clkcnt < 6 };
	always @(posedge clk) 
		if (rxedge) ta <= td[5:0];
	wire sync_b = td == 'h96;
	wire sync_m = td == 'ha5;
	wire sync_w = td == 'h84;
	wire sync = sync_b | sync_m | sync_w;
	reg lr, bi;
	always @(posedge clk) 
		if (rxedge & sync) lr <= sync_w;
	always @(posedge clk) 
		if (rxedge) bi <= ~sync & (bi ^ td[0]);
	wire shift = bi | ~td[0];
	reg [5:0] bitcnt;
	always @(posedge clk)
		if (rxedge) 
			if (sync) bitcnt <= 0;
			else if (shift) bitcnt <= bitcnt + 1;
	reg [6:0] volcnt;
	wire t0 = volcnt[1:0] == 0;
	reg [`MSB:0] data;
	always @(posedge clk)
		if (volcnt >= 'h3c) begin
			if (rxedge & ~sync & shift) data <= { td[0], data[`MSB:1] };
		end
		else if (t0) data <= { data[`MSB], data[`MSB:1] };
	always @(posedge clk) 
		if (rxedge & ~sync & shift & bitcnt == 23) volcnt <= 0;
		else if (volcnt != 'h44) volcnt <= volcnt + 1;
	reg [`MSB+1:0] acc;
	wire voldata;
	always @(posedge clk) 
		if (volcnt[6] | mute) acc <= 0;
		else if (t0 & voldata) acc <= acc + { data[`MSB], data };
	reg [`MSB:0] lo, ro;
	always @(posedge clk) 
		if (volcnt == 'h3f) 
			if (lr) lo <= { ~acc[`MSB+1], acc[`MSB:1] };
			else ro <= { ~acc[`MSB+1], acc[`MSB:1] };
	reg [8:0] vols;
	always @(posedge clk) 
		if (volcnt == 'h1c) vols <= vol;
		else if (t0) vols <= { vols[7:0], 1'b0 };
	curve curve(.program(1'b0), .erase(1'b0), .oscena(1'b0),
		.arclk(mclk), .arshft(1'b1), .ardin(vols[8]),
		.drclk(mclk), .drshft(~volcnt[6]), .drdin(1'b0), .drdout(voldata));
	assign timing = rxedge & sync_b;
	assign mclk0 = volcnt[1];
endmodule
